/***************************************************************************
                          pictureplugin.cpp  -  description
                             -------------------
    begin                : Son Apr 14 2002
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pictureplugin.h"

// QT includes
#include <qimage.h>

// KDE includes
#include <kapplication.h>
#include <klocale.h>

PicturePlugin::PicturePlugin()
    : FilePlugin( 0 )
{
    keys.append( "resolution" );
    keys.append( "xres" );
    keys.append( "yres" );
    keys.append( "bitdepth" );   
    setupKeys();
    
    m_icon = "image";
}

const QString PicturePlugin::getName() const
{
    return i18n("Picture Plugin");
}

const QString PicturePlugin::getAccelName() const
{
    return i18n("P&icture Plugin");
}

const QString PicturePlugin::getPattern() const
{
    return "pic";
}

QString PicturePlugin::processFile( BatchRenamer* b, int i, QString token, int )
{
    QString resolution;
    QString xres;
    QString yres;
    QString bitdepth;
    
    QString filename = BatchRenamer::buildFilename( &b->files()[i].src );

    token = token.lower();
       
    /*
     * Check if we have something cached for this file
     */
    if( cache.contains( filename + "::" + token ) )
        return cache[filename + "::" + token ];
    
    QImage img( filename );
    if( img.isNull() )
        return QString::null;

    resolution = QString( "%1x%2" ).arg(img.width()).arg(img.height());
    xres = QString::number( img.width() );
    yres = QString::number( img.height() );
    bitdepth = QString::number( img.depth() );

    if( cache.count() >= CACHE_MAX )
        cache.remove( cache.begin() );

    QString ret = QString::null;
    
    if( token == getPattern() + "resolution" )
        ret = resolution;
    else if( token == getPattern() + "xres" )
        ret = xres;
    else if( token == getPattern() + "yres" )
        ret = yres;
    else if( token == getPattern() + "bitdepth" )
        ret = bitdepth;

    cache.insert( filename + "::" + token, ret );    
    return ret;
}


